// Information used by resource files
#define INFO_PRODUCT_VERSION_LIST 0,11,0
#define INFO_PRODUCT_VERSION_STRING "0.11.0.0"
#define INFO_FILE_VERSION_LIST INFO_PRODUCT_VERSION_LIST,0
#define INFO_FILE_VERSION_STRING INFO_PRODUCT_VERSION_STRING "." 0
