



if not exist %LIBRARY_INC%\\log4cxx\\log4cxx.h exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %LIBRARY_BIN%\\log4cxx.dll exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %LIBRARY_LIB%\\log4cxx.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
